//+------------------------------------------------------------------+
//|                                                                  |
//|  R-DigitalFilter v.1.0                                           |
//|  Copyright  2008 Riccardo "Rich" Cap                            |
//|  email: richcap.rc@gmail.com                                     |
//|                                                                  |
//|                                                                  |
//|  At every Bar beginning  the price series are lowpass FIR        |
//|  filtered with Parks-McClellan algorythm                         |
//|  Input parameters are: cutoff periods (passband and stopband)    |
//|  stopband attenuation and passband ripple                        |
//|  Filter can be applied to Open, Low, High or Close price series  | 
//|                                                                  |
//|  Permission to use, copy, modify, and distribute this software   |
//|  for any purpose without fee is hereby granted, provided that    |
//|  this entire notice is included in all copies of any software    |
//|  which is or includes a copy or modification of this software    |
//|  and in all copies of the supporting documentation for such      |
//|  software.                                                       |
//|  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS    |
//|  OR IMPLIED WARRANTY.  IN PARTICULAR, NEITHER THE AUTHOR DOES    |
//|  NOT MAKE ANY REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING  |
//|  THE MERCHANTABILITY OF THIS SOFTWARE OR ITS FITNESS FOR ANY     |
//|  PARTICULAR PURPOSE.                                             |
//|                                                                  |
//+------------------------------------------------------------------+


#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Red

//---- buffers
double Buffer1[];

//---

extern double P1=130;                  // cutoff period P1
extern double D1=70;                   // cutoff period D1 (transition)
extern double A1 = 40.0;               // stopband attenuation in dB
extern double ripple = 0.8;            // passband ripple in dB
extern int BarShift=0;
extern int Mode=3;                     // Apply to 0=Open, 1=Low, 2=High, 3=Close (default)
   
// FIR
double FIR[];
double bands[20];
double des[10];
double weight[10];
int filterOrder=0;
int filterSize=0;

static datetime prevtime=0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
     
   //---- indicators
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,Buffer1);
   
   // Create Filter Specs   
   double fp = 1.0/P1;           // cutoff frequency passband (corresponding to cutoff period P1)
   double fs = 1.0/D1;           // cutoff frequency stopband (corresponding to cutoff period D1) 
   double deltap=0.5*(1.0 - MathPow(10.0, -0.05*ripple));
   double deltas = MathPow(10.0, -0.05*A1);
   
   bands[0]=0.0;                 // Beginning of first band
   bands[1]= fp;                 // end of first band
   des[0]=1.0;                   // gain in first band
   weight[0]=1.0;                // weight in first band
   
   bands[2]=fs;
   bands[3]=1.0/2.0;
   des[1]=0.0;                   // gain in second band (stopband)  
   weight[1]=deltap/deltas;      // weight in second band (stopband): it is 1.0 if deltap=deltas
   
   /*
   bands[4]=1.0/20.0;
   bands[5]=0.5;
   des[2]=0.001;                 // gain in third band (stopband)  -40dB
   weight[2]=1.0;                // weight in second band (stopband)
   */
   
   // Estimate filter order 
   filterOrder=PMcCl_filterOrd(deltap,deltas,fp,fs);   
   ArrayResize(FIR,filterOrder);
   
   
   // Create Filter Coefficients
   if (!DF( FIR, filterOrder, 2, bands, des, weight, 1))     // Type of filter BANDPASS with 2 bands
      Print("Unable to create digital filter");
   
   filterSize=filterOrder/2+filterOrder%2;
   Print("Number of coefficients: ",filterSize," - Filter Order", filterOrder);
   double gain=0.0;
   for (int j=0; j<filterSize; j++) 
      gain+=FIR[j];                       
        
   Print("Gain: ",gain);
   
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   //----
   int handle;
   handle=FileOpen("DF_2_Output.csv", FILE_CSV|FILE_WRITE, ';');
   if(handle>0)
    {
      for(int i=0; i<filterOrder-1; i++)
      {
         FileWrite(handle, "Filter coefficient ",i," :", DoubleToStr(FIR[i],18));
      }
     FileClose(handle);
     Print("File  saved ");
    }
   else
    {
      Print("File  not found, the last error is ", GetLastError());
      return(false);
    }
   //----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{ 
   int i, j, counted_bars=IndicatorCounted();
   double value;

   if(!newPeriodOpening())
   {
      //Print("Not the beginning of a new period");
      return(0);  // Not the beginning of a new period
   } 

   if (filterOrder>0) 
   {
      
      if(Bars<=filterSize) return(0);

      i=Bars-filterSize-1;
      if (counted_bars>=filterSize) 
         i=Bars-counted_bars-1;
        
      while(i>=BarShift)
      {
         Buffer1[i-BarShift]=0;
         for (j=0; j<filterSize; j++) 
         {
            switch (Mode)
            {
               case MODE_CLOSE: 
                  value = Close[i+j];
                  break;
               case MODE_HIGH: 
                  value = High[i+j];
                  break;
               case MODE_LOW: 
                  value = Low[i+j];
                  break;
               case MODE_OPEN: 
                  value = Open[i+j];
                  break;
               default:  
                  value = Close[i+j];
                  break;
             }
            
            Buffer1[i-BarShift]=Buffer1[i-BarShift]+FIR[j]*value;   
         }
         i--;
      }
   } 
   else 
      Print("Indicator cannot run.");
      
   return(0);
}
  
  
bool newPeriodOpening()
  {
   if(prevtime == Time[0]) return(false);
   prevtime = Time[0];
   return(true);
  }
  
  
  
/**************************************************************************
 * Parks-McClellan algorithm for FIR filter design (C version)
 *-------------------------------------------------
 *  Copyright (c) 1995,1998  Jake Janovetz (janovetz@uiuc.edu)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *************************************************************************/



#define M_PI	3.1415926535897932384626433832795029

#define BANDPASS       1
#define DIFFERENTIATOR 2
#define HILBERT        3

#define NEGATIVE       0
#define POSITIVE       1

#define GRIDDENSITY    16
#define MAXITERATIONS  90





/*
 * DF (aka remez)
 *=======
 * Calculates the optimal (in the Chebyshev/minimax sense)
 * FIR filter impulse response given a set of band edges,
 * the desired reponse on those bands, and the weight given to
 * the error in those bands using the Parks-McClellan exchange algorithm.
 *
 *
 * DF returns as many coefficients (impulse response) as specified by parameter "numtaps".
 *
 * "bands" is a vector of band edge pairs (start - end), which specify the start and end of
 *    the "numband" bands in the filter specification. These must be non-overlapping and 
 *    sorted in increasing order. Only values between 0 (0 Hz) and 0.5 (the Nyquist frequency)
 *    are allowed.
 * "des" specifies the desired gain in these bands.
 * "weight" can be used to give each band a different weight. If absent, a vector of ones is used.
 * "type" is any of the exported constants BANDPASS, DIFFERENTIATOR or HILBERT and can be used to 
 *    select various design types (use BANDPASS until this is documented ;)
 *
 */
bool DF(
   double& h[],         // Impulse response of final filter [numtaps] (OUTPUT)
   int numtaps,         // Number of filter coefficients (INPUT)
   int numband,         // Number of bands in filter specification (INPUT)
   double& bands[],     // User-specified band edges [2 * numband] (INPUT)
   double& des[],       // User-specified band responses [numband] (INPUT)
   double& weight[],    // User-specified error weights [numband] (INPUT)
   int type)            // Type of filter BANDPASS-DIFFERENTIATOR-HILBERT (INPUT)
{
   double Grid[], W[], D[], E[];
   int    i, iter, gridsize, r, Ext[];
   double taps[], c;
   double x[], y[], ad[];
   int    symmetry;

   if (type == BANDPASS)
      symmetry = POSITIVE;
   else
      symmetry = NEGATIVE;

   r = numtaps/2;                               // number of extrema 
   if ((numtaps%2==1) && (symmetry == POSITIVE))               
      r++;
   

   // Predict dense grid size in advance for memory allocation
   //   .5 is so we round up, not truncate
   gridsize = 0;
   for (i=0; i<numband; i++)
   {
      gridsize += MathRound(2.0*r*GRIDDENSITY*(bands[2*i+1] - bands[2*i]));  
   }
   if (symmetry == NEGATIVE)
   {
      gridsize--;
   }
   

   // Dynamically allocate memory for arrays with proper sizes

   ArrayResize(Grid,gridsize);
   ArrayResize(D,gridsize);
   ArrayResize(W,gridsize);
   ArrayResize(E,gridsize);
   
   ArrayResize(Ext,r+1);
   ArrayResize(taps,r+1);
   ArrayResize(x,r+1);
   ArrayResize(y,r+1);
   ArrayResize(ad,r+1);
   
   ArrayInitialize(Grid,0.0);
   ArrayInitialize(D,0.0);
   ArrayInitialize(W,0.0);
   ArrayInitialize(E,0.0);
   ArrayInitialize(Ext,0);
   ArrayInitialize(taps,0.0);
   ArrayInitialize(x,0.0);
   ArrayInitialize(y,0.0);
   ArrayInitialize(ad,0.0);


   // Create dense frequency grid

   CreateDenseGrid(r, numtaps, numband, bands, des, weight,
                   gridsize, Grid, D, W, symmetry);
   InitialGuess(r, Ext, gridsize);


   // For Differentiator: (fix grid)
   if (type == DIFFERENTIATOR)
   {
      for (i=0; i<gridsize; i++)
      {
         if (D[i] > 0.0001)
            W[i] = W[i]/Grid[i];
      }
   }


   // For odd or Negative symmetry filters, alter the
   // D[] and W[] according to Parks McClellan
   if (symmetry == POSITIVE)
   {
      if (numtaps%2 == 0)
      {
         for (i=0; i<gridsize; i++)
         {
            c = MathCos(M_PI * Grid[i]);
            D[i] /= c;
            W[i] *= c; 
         }
      }
   }
   else
   {
      if (numtaps%2 == 1)
      {
         for (i=0; i<gridsize; i++)
         {
            c = MathSin(2.0 * M_PI * Grid[i]);
            D[i] /= c;
            W[i] *= c;
         }
      }
      else
      {
         for (i=0; i<gridsize; i++)
         {
            c = MathSin(M_PI * Grid[i]);
            D[i] /= c;
            W[i] *= c;
         }
      }
   }


   // Perform the Remez Exchange algorithm
   for (iter=0; iter<MAXITERATIONS; iter++)
   {
      CalcParms(r, Ext, Grid, D, W, ad, x, y);
      CalcError(r, ad, x, y, gridsize, Grid, D, W, E);
      Search(r, Ext, gridsize, E);
      if (isDone(r, Ext, E)==1)
         break;
   }
   if (iter == MAXITERATIONS)
   {
      Print("reached maximum iteration count, results may be bad.\n");
      return (FALSE);
   }

   CalcParms(r, Ext, Grid, D, W, ad, x, y);
   
   // Find the 'taps' of the filter for use with Frequency
   // Sampling.  If odd or Negative symmetry, fix the taps
   // according to Parks McClellan
   for (i=0; i<=numtaps/2; i++)
   {
      if (symmetry == POSITIVE)
      {
         if (numtaps%2 == 1)
            c = 1;
         else
            c = MathCos(M_PI  * (1.0*i)/numtaps);           
      }
      else
      {
         if (numtaps%2 == 1)
            c = MathSin(2.0 * M_PI * (1.0*i)/numtaps);      
         else
            c = MathSin(M_PI  * (1.0*i)/numtaps);           
      }
      taps[i] = ComputeA((1.0*i)/numtaps, r, ad, x, y)*c;   
   }

   // Frequency sampling design with calculated taps
   FreqSample(numtaps, taps, h, symmetry);
   return (TRUE);

}


/*
 * Parks-McClellan Filter order estimator using Hermann-Rabiner-Chans's Formula (only for 2 band filter)
 *
 *  Filter Specs
 *  ------------
 *  Example
 *  double ripple = 0.8;          // passband ripple in dB
 *  double atten = 40.0;          // stopband attenuation in dB
 *  double fp = 1.0/P1;           // cutoff frequency passband (corresponding to cutoff period P1)
 *  double fs = 1.0/D1;           // cutoff frequency stopband (corresponding to cutoff period D1) 
 *  double deltap=0.5*(1.0 - MathPow(10.0, -0.05*ripple));
 *  double deltas = MathPow(10.0, -0.05*atten);
 * 
 *  bands[0]=0.0;                 // Beginning of first band
 *  bands[1]= fp;                 // end of first band
 *  des[0]=1.0;                   // gain in first band
 *  weight[0]=1.0;                // weight in first band
 * 
 *  bands[2]=fs;
 *  bands[3]=1.0/2.0;
 *  des[1]=0.0;                   // gain in second band (stopband)  
 *  weight[1]=deltap/deltas;      // weight in second band (stopband): it is 1.0 if deltap=deltas
 * 
 */
int PMcCl_filterOrd(
   double deltap,       //  Ripple for first band (passband, for a lowpass filter)
   double deltas,       //  Ripple for second band (stopband, for a lowpass filter)
   double fp,           //  Cutoff frequency for first band (startof transition band)
   double fs)           //  Cutoff frequency for second band (end of transition band)
{
   double a1=0.00539;
   double a2=0.07114;
   double a3=-0.4761;
   double a4=-0.00266;
   double a5=-0.5941;
   double a6=-0.4278;
   double b1=11.01217;
   double b2=0.51244; 
   
   if (MathAbs(deltas) > MathAbs (deltap))
   {
      Print ("First band ripple must be equal or greater than second band ripple");
      return (-1);
   }

   // Metatrader has only Ln (MathLog) but you know: Log10(x)=Ln(x)/Ln(10)
   double Dinf=(a1 * MathPow(MathLog(deltap)/MathLog(10),2) + a2 * MathLog(deltap)/MathLog(10) + a3);
   Dinf *= MathLog(deltas)/MathLog(10);
   Dinf += ( a4 * MathPow(MathLog(deltap)/MathLog(10),2) + a5 * MathLog(deltap)/MathLog(10) + a6 );

   double F = b1 + b2 * (MathLog(deltap)/MathLog(10) - MathLog(deltas)/MathLog(10));
   
   int N = Dinf / (fs - fp) - F * (fs - fp) + 1;
   return (N);
}


/*
 * CreateDenseGrid
 *=================
 * Creates the dense grid of frequencies from the specified bands.
 * Also creates the Desired Frequency Response function (D[]) and
 * the Weight function (W[]) on that dense grid
 */
void CreateDenseGrid(
   int r,               // 1/2 the number of filter coefficients
   int numtaps,         // Number of taps in the resulting filter
   int numband,         // Number of bands in user specification
   double& bands[],     // User-specified band edges [2*numband]
   double& des[],       // Desired response per band [numband]
   double& weight[],    // Weight per band [numband]
   int gridsize,        // Number of elements in the dense frequency grid 
   double& Grid[],      // Frequencies (0 to 0.5) on the dense grid [gridsize] (OUTPUT)
   double& D[],         // Desired response on the dense grid [gridsize] (OUTPUT)
   double& W[],         // Weight function on the dense grid [gridsize] (OUTPUT)
   int symmetry)        // Symmetry of filter - used for grid check
{
   int i, j, k, band;
   double delf, lowf, highf;

   delf = 0.5/(GRIDDENSITY*r);


   // For differentiator, hilbert,
   // symmetry is odd and Grid[0] = max(delf, band[0])
   if ((symmetry == NEGATIVE) && (delf > bands[0]))
      bands[0] = delf;

   j=0;
   for (band=0; band < numband; band++)
   {
      Grid[j] = bands[2*band];
      lowf = bands[2*band];
      highf = bands[2*band + 1];
      k = MathRound((highf - lowf)/delf);   /* .5 for rounding */   // is OK?? Now it is!
      for (i=0; i<k; i++)
      {
         D[j] = des[band];
         W[j] = weight[band];
         Grid[j] = lowf;
         lowf += delf;
         j++;
      }
      Grid[j-1] = highf;
   }


   // Similar to above, if odd symmetry, last grid point can't be .5
   //  - but, if there are even taps, leave the last grid point at .5
   if ((symmetry == NEGATIVE) &&
       (Grid[gridsize-1] > (0.5 - delf)) &&
       ((numtaps % 2)!=0))
   {
      Grid[gridsize-1] = 0.5-delf;
   }
}


/*
 * InitialGuess
 *==============
 * Places Extremal Frequencies evenly throughout the dense grid.
 *
 *
 */
void InitialGuess(
   int r,               // 1/2 the number of filter coefficients
   int& Ext[],          // Number of elements in the dense frequency grid
   int gridsize)        // Extremal indexes to dense frequency grid [r+1] (OUTPUT)
{
   int i;

   for (i=0; i<=r; i++)
      Ext[i] = (i * (gridsize-1)) / r;   // check -  Seems OK but I did put brakets just in case
}


/*
 * CalcParms
 */
void CalcParms(
   int r,               // 1/2 the number of filter coefficients
   int& Ext[],          // Extremal indexes to dense frequency grid [r+1]
   double& Grid[],      // Frequencies (0 to 0.5) on the dense grid [gridsize]
   double& D[],         // Desired response on the dense grid [gridsize]
   double& W[],         // Weight function on the dense grid [gridsize]
   double& ad[],        // 'b' in Oppenheim & Schafer [r+1] (OUTPUT)
   double& x[],         // [r+1] (OUTPUT)
   double& y[])         // 'C' in Oppenheim & Schafer [r+1] (OUTPUT)
{
   int i, j, k, ld;
   double sign, xi, delta, denom, numer;


   // Find x[]
   for (i=0; i<=r; i++)
   {  x[i] = MathCos(2.0 * M_PI * Grid[Ext[i]]);   }
   

   // Calculate ad[]  - Oppenheim & Schafer eq 7.132
   ld = (r-1)/15 + 1;         /* Skips around to avoid round errors */   //check
   for (i=0; i<=r; i++)
   {
       denom = 1.0;
       xi = x[i];  
       for (j=0; j<ld; j++)
       {
          for (k=j; k<=r; k+=ld)
             if (k != i)
                denom *= 2.0*(xi - x[k]);
       }
       if (MathAbs(denom)<0.00001)
          denom = 0.00001;
       ad[i] = 1.0/denom;
   }
 
   // Calculate delta  - Oppenheim & Schafer eq 7.131
   numer = 0;
   denom = 0;
   sign = 1;
   for (i=0; i<=r; i++)
   {
      numer += ad[i] * D[Ext[i]];
      denom += sign * ad[i]/W[Ext[i]];
      sign = -sign;     
   }
   if (MathAbs(denom)<0.00001)              // added control for error below
          denom = 0.00001;
   delta = numer/denom;                     // divide by zero error for r=98 (order=195) 
   sign = 1;
   

   // Calculate y[]  - Oppenheim & Schafer eq 7.133b
   for (i=0; i<=r; i++)
   {
      y[i] = D[Ext[i]] - sign * delta/W[Ext[i]];
      sign = -sign;
   }
}


/*
 * ComputeA
 *==========
 * Using values calculated in CalcParms, ComputeA calculates the
 * actual filter response at a given frequency (freq).  Uses
 * eq 7.133a from Oppenheim & Schafer.
 *
 * Returns double value of A[freq]
 */
double ComputeA(
   double freq,      // Frequency (0 to 0.5) at which to calculate A
   int r,            // 1/2 the number of filter coefficients
   double& ad[],     // 'b' in Oppenheim & Schafer [r+1]
   double& x[],      // [r+1]
   double& y[])      // 'C' in Oppenheim & Schafer [r+1]
{
   int i;
   double xc, c, denom, numer;

   denom = 0;
   numer = 0;
   xc = MathCos(2.0 * M_PI * freq);
   for (i=0; i<=r; i++)
   {
      c = xc - x[i];
      if (MathAbs(c) < 0.0000001)
      {
         numer = y[i];
         denom = 1;
         break;
      }
      c = ad[i]/c;
      denom += c;
      numer += c*y[i];
   }
   return (numer/denom);
}


/*
 * CalcError
 *===========
 * Calculates the Error function from the desired frequency response
 * on the dense grid (D[]), the weight function on the dense grid (W[]),
 * and the present response calculation (A[])
 *
 */
void CalcError(
   int r,            // 1/2 the number of filter coefficients
   double& ad[],     //
   double& x[],      //
   double& y[],      // 
   int gridsize,     // Number of elements in the dense frequency grid
   double& Grid[],   // Frequencies on the dense grid [gridsize]
   double& D[],      // Desired response on the dense grid [gridsize]
   double& W[],      // Weight function on the desnse grid [gridsize]
   double& E[])      // Error function on dense grid [gridsize]  (OUTPUT)
{
   int i;
   double A;

   for (i=0; i<gridsize; i++)
   {
      A = ComputeA(Grid[i], r, ad, x, y);
      E[i] = W[i] * (D[i] - A);
   }
}

/*
 * Search
 *========
 * Searches for the maxima/minima of the error curve.  If more than
 * r+1 extrema are found, it uses the following heuristic (thanks
 * Chris Hanson):
 * 1) Adjacent non-alternating extrema deleted first.
 * 2) If there are more than one excess extrema, delete the
 *    one with the smallest error.  This will create a non-alternation
 *    condition that is fixed by 1).
 * 3) If there is exactly one excess extremum, delete the smaller
 *    of the first/last extremum
 */
void Search(
   int r,            // 1/2 the number of filter coefficients
   int& Ext[],       // Indexes to Grid[] & new indexes to extremal frequencies [r+1] (INPUT and OUTPUT)
   int gridsize,     // Number of elements in the dense frequency grid
   double& E[])      // Array of error values.  [gridsize]
{
   int i, j, k, l, extra;     /* Counters */
   int up, alt;
   int foundExt[];            /* Array of found extremals */


   // Allocate enough space for found extremals.
   ArrayResize(foundExt,2*r);
   ArrayInitialize(foundExt,0.0);
   k = 0;

   // Check for extremum at 0.
   if (((E[0]>0.0) && (E[0]>E[1])) ||
       ((E[0]<0.0) && (E[0]<E[1])))
       {
         foundExt[k] = 0;
         k++;
       }

   // Check for extrema inside dense grid
   for (i=1; i<gridsize-1; i++)
   {
      if (((E[i]>=E[i-1]) && (E[i]>E[i+1]) && (E[i]>0.0)) ||
          ((E[i]<=E[i-1]) && (E[i]<E[i+1]) && (E[i]<0.0)))   
       {
         foundExt[k] = i;
         k++;
       }
   }
   
   // Check for extremum at 0.5
   j = gridsize-1;
   if (((E[j]>0.0) && (E[j]>E[j-1])) ||
       ((E[j]<0.0) && (E[j]<E[j-1])))
      {
         foundExt[k] = j;
         k++;
      }

   // Remove extra extremals
   extra = k - (r+1);

   while (extra > 0)
   {
      if (E[foundExt[0]] > 0.0)
         up = 1;                /* first one is a maxima */
      else
         up = 0;                /* first one is a minima */

      l=0;
      alt = 1;
      for (j=1; j<k; j++)
      {
         if (MathAbs(E[foundExt[j]]) < MathAbs(E[foundExt[l]]))
            l = j;               /* new smallest error. */
         if ((up) && (E[foundExt[j]] < 0.0))
            up = 0;             /* switch to a minima */
         else if ((!up) && (E[foundExt[j]] > 0.0))
            up = 1;             /* switch to a maxima */
         else
	      { 
            alt = 0;
            break;              /* Ooops, found two non-alternating */
         }                      /* extrema.  Delete smallest of them */
      }  /* if the loop finishes, all extrema are alternating */

      // If there's only one extremal and all are alternating,
      // delete the smallest of the first/last extremals.
      if ((alt) && (extra == 1))
      {
         if (MathAbs(E[foundExt[k-1]]) < MathAbs(E[foundExt[0]]))
            l = foundExt[k-1];   /* Delete last extremal */
         else
            l = foundExt[0];     /* Delete first extremal */
      }

      for (j=l; j<k; j++)        /* Loop that does the deletion */
      {
         foundExt[j] = foundExt[j+1];
      }
      k--;
      extra--;
   }

   for (i=0; i<=r; i++)
   {
      Ext[i] = foundExt[i];       /* Copy found extremals to Ext[] */
   }

}


/*
 * FreqSample
 *============
 * Simple frequency sampling algorithm to determine the impulse
 * response h[] from A's found in ComputeA
 */
void FreqSample(
   int N,            // Number of filter coefficients
   double& A[],      // Sample points of desired response [N/2]
   double& h[],      // Impulse Response of final filter [N] (OUTPUT)
   int symm)         // Symmetry of desired filter
{
   int n, k;
   double x, val, M;

   M = (N-1.0)/2.0;
   if (symm == POSITIVE)
   {
      if (N%2 != 0)                                   
      {
         for (n=0; n<N; n++)
         {
            val = A[0];
            x = 2.0 * M_PI * (n - M)/N;
            for (k=1; k<=M; k++)
               val += 2.0 * A[k] * MathCos(x*k);
            h[n] = val/N;
         }
         
      }
      else
      {
         for (n=0; n<N; n++)
         {
            val = A[0];
            x = 2.0 * M_PI * (n - M)/N;
            for (k=1; k<=(N/2-1); k++)
               val += 2.0 * A[k] * MathCos(x*k);
            h[n] = val/N;
         }
         
      }
      
      passbandNormalisation( N, h )     ;
      
   }
   else
   {
      if (N%2 != 0)                                  
      {
         for (n=0; n<N; n++)
         {
            val = 0;
            x = 2.0 * M_PI * (n - M)/N;
            for (k=1; k<=M; k++)
               val += 2.0 * A[k] * MathSin(x*k);
            h[n] = val/N;
         }
      }
      else
      {
          for (n=0; n<N; n++)
          {
             val = A[N/2] * MathSin(M_PI * (n - M));
             x = 2.0 * M_PI * (n - M)/N;
             for (k=1; k<=(N/2-1); k++)
                val += 2.0 * A[k] * MathSin(x*k);
             h[n] = val/N;
          }
      }
   }
   
}

/*
 * isDone
 *========
 * Checks to see if the error function is small enough to consider
 * the result to have converged.
 *
 * Returns 1 if the result converged
 * Returns 0 if the result has not converged
 */
int isDone(
   int r,            // 1/2 the number of filter coeffiecients
   int& Ext[],       // Indexes to extremal frequencies [r+1]
   double& E[])      // Error function on the dense grid [gridsize]
{
   int i;
   double min, max, current;

   min = MathAbs(E[Ext[0]]);
   max = min;
   for (i=1; i<=r; i++)
   {
      current = MathAbs(E[Ext[i]]);
      if (current < min)
         min = current;
      if (current > max)
         max = current;
   }
   if (((max-min)/max) < 0.0001)
      return (1);
   return (0);
}

/*
 * Normalize, halve and reverse symmetric filter to get it in the right way to
 * use in an indicator (minimize lag and perform less numeric operations)
 */
void passbandNormalisation(
   int N,            // Number of filter coefficients
   double& h[])     // Impulse Response of final filter [N] (OUTPUT)
{
      int r=N/2, length=r;
      double gain=0.0, buf;
      
      // what's the gain at 0 freq?
      for (int n=0; n<r; n++)
      {
         gain+=2*h[n];
      }
      if (N%2 != 0)              // what if N is odd ?
      {
         gain+=h[r];             // add center coefficient to the gain
         length=r+1;             // length of the filter is r+1
         h[r]=h[r]/gain;         // normalize last (first) coefficient (it results in about a half)
      }  
      // normalize first r coefficients         // check if correct forn passband normalisation
      for (n=0; n<r; n++)
      {
         h[n]=2*h[n]/gain;
      }
      
      // zero all other coefficients
      for (n=length; n<N; n++)
      {   
         h[n]=0.0;
      }
     
         
      // reverse coefficients 
      int halfLength=length/2+length%2;
      //Print ("Half Length:",halfLength,"- length:",length);
      for (n=0; n<halfLength; n++)  
      {
         buf=h[n];
         h[n]=h[length-1-n];
         h[length-1-n]=buf;
      }
      
}
//-------------------------------------------------------------------+